
# $Id: simulations.R,v 1.12 2005/12/08 18:30:32 jon Exp $
# $Source: /projects/starl/cvs/dp-eb/src/simulations.R,v $

###
### Copyright (c) 2004, 2005, David M. Blei and Jon D. McAuliffe.
### This software is distributed under the terms of the GNU
### General Public License.
###



stopifnot(Sys.getenv("DPEB_LIB") != "");
source(paste(Sys.getenv("DPEB_LIB"), "/dp-model.R", sep = ""));

g0.lst <- list(
               ## standard normal
               list(rg0 = rnorm,
                    dg0 = dnorm,
                    sd = 1),

               ## t(3)
               list(rg0 = function(n) rt(n, df = 3),
                    dg0 = function(x) dt(x, df = 3),
                    sd = sqrt(3)),

               ## density of -log(Z), Z ~ chisq(df = 1).
               list(rg0 = function(n) -log(rchisq(n, df = 1)),
                    dg0 = function(x)
                      suppressWarnings(
                        ifelse(abs(x) > 100, 0,
                               exp(-x) * dchisq(exp(-x), df = 1))
                      ),
                    sd = sqrt(trigamma(0.5))),

               ## an equal mixture of two t(5)'s, modes at -3 and 3.
               list(rg0 = function(n)
                      rt(n, df = 5) +
                      ifelse(sample(2, n, replace = TRUE) == 1, -3, 3),
                    dg0 = function(x)
                      (dt(x-3, df = 5) + dt(x+3, df = 5)) / 2,
                    sd = sqrt(32/3))
               );

## default parameters:

## n = 500
## m = 10
## b = 100
## lag = 1
## burn.in = 100
## num.rounds = 200

sim <- function(alpha, g0, sd.scale, sim.dir,
                n=500, m=10, b=200, lag=1, burn.in=100, num.rounds=200)
{
  rg0 <- g0$rg0;
  dg0 <- g0$dg0;
  g0.sd <- g0$sd;

  rdata <- function(n, thetas) rnorm(n, mean = thetas, sd = g0.sd/sd.scale);
  ddata <- function(y, thetas) dnorm(y, mean = thetas, sd = g0.sd/sd.scale);

  simulation(n = n, m = m, b = b, lag = lag, burn.in = burn.in,
             num.rounds = num.rounds, alpha = alpha,
             rg0=rg0, dg0=dg0, rdata=rdata, ddata=ddata,
             seed = as.integer(Sys.time()),
             sim.dir = sim.dir);
}


bayes.gibbs <- function(g0, sd.scale, sim.dir)
{
  load(paste(sim.dir,"/global-state.Rdata", sep=""), envir=.GlobalEnv);
  set.seed(seed);
  data.sd <- g0$sd/sdscale;
  state.lst <- bayesian.inference(dataset, data.sd);
  save(state.lst, file=paste(sim.dir,"/bayes-samples.Rdata", sep=""))
}

true.gibbs <- function(sim.dir)
{
  load(paste(sim.dir,"/global-state.Rdata", sep=""), envir=.GlobalEnv);
  set.seed(seed);
  gibbs.summary <<- array(dim = c(B, LAG, 2));

  eb.results  <<- list();
  eb.results$alpha.hat <<- alpha;
  eb.results$dg0.hat <<- dg0;
  eb.results$rg0.hat <<- rg0;
  
  gsamples <<-
      lapply(1:B, function(i) list(classes     = numeric(N),
                                   theta.stars = numeric(N),
                                   counts      = numeric(N)));
  
  rg0.hat    <<- eb.results$rg0.hat;
  alpha.hat  <<- eb.results$alpha.hat;
  
  first.state(B);
  collect.B.samples();
  
  save(eb.results, gsamples, file=paste(sim.dir, "/true-gibbs.Rdata", sep=""));
} 
